import unittest
import tifffile as tif
import numpy as np

from pathlib import Path
import matplotlib.pyplot as plt

import vodex as vx
from plots import *

project_dir = "D:/Code/repos/numan/notebooks/data/v1.x.x/2vs3vs5/processed"


class TestLabelPlotter(unittest.TestCase):
    experiment = vx.Experiment.load(Path(project_dir, "experiment_raw.db"))

    def test_init(self):
        lp = LabelPlotter(self.experiment, "number")
        names = ["b", "d2", "d3", "d5"]
        values = [0, 0, 0, 0, 0, 0, 0, 2,  # 6,7 - 8
                  0, 0, 0, 0, 0, 1,  # 12,13 - 14
                  0, 0, 0, 0, 0, 0, 3,  # 19,20 - 21
                  0, 0, 0, 0, 0, 0, 1,  # 26,27 - 28
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 2,  # 36,37 - 38
                  0, 0, 0, 0, 0, 1,  # 42, 43 - 44
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 3,  # 52, 53 - 54
                  0, 0, 0, 0, 0, 0, 2,  # 59, 60 - 61
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 3]  # 69, 70
        self.assertListEqual(names, lp.names)
        self.assertListEqual(values, lp.values)

    def test_plot_labels(self):
        lp = LabelPlotter(self.experiment, "number")
        lp.plot_labels(show_plot=True)
        lp.plot_labels(forward_shift=1, show_plot=True)
        lp.plot_labels(time_points=[6, 7, 8, 12, 13, 14, 19, 20, 21],
                       show_plot=True)
        lp.plot_labels(forward_shift=1,
                       time_points=[6, 7, 8, 12, 13, 14, 19, 20, 21],
                       show_plot=True)


class TestSignalPlotter(unittest.TestCase):
    experiment = vx.Experiment.load(Path(project_dir, "experiment_raw.db"))
    spots = Spots.from_json(f"{project_dir}/spots/signals/spots_SvB_max.json")

    # initialise the signal plotter with DFF signal
    SLIDING_WINDOW = 15  # in volumes
    signals = spots.get_group_signals(spots.groups["sig2vB"]).as_dff(SLIDING_WINDOW)
    s_plotter = SignalPlotter(signals, experiment, "number")

    def test_prepare_cycle(self):
        trace_id = 0
        trace = self.s_plotter.get_trace(trace_id)
        trace = self.s_plotter.prepare_cycle(trace)
        self.assertTupleEqual((18, 71), trace.shape)

    def test_plot_cycles(self):
        trace_id = 0
        ax_limits = self.s_plotter.plot_cycles(None, trace_id, show_plot=True)
        self.s_plotter.plot_cycles(None, trace_id, show_plot=True, forward_shift=3)
        self.assertAlmostEqual(
            [-0.5, 70.5, -0.04479955511212163, 0.07627925625219258],
            ax_limits)

    def test_plot_psh(self):
        trace_id = 0
        labels = ["d2", "d3", "d5"]
        padding = [-2, -1, 0, 1, 2, 3, 4]
        _ = self.s_plotter.plot_psh(None, trace_id, labels, padding, show_plot=True)
        padding = [0]
        _ = self.s_plotter.plot_psh(None, trace_id, labels, padding,
                                    split=False, plot_individual=False, show_plot=True)

    def test_make_psh_figure(self):
        trace_ids = [0, 1, 2, 3, 4]
        labels = ["d2", "d3", "d5"]
        padding = [-2, -1, 0, 1, 2, 3, 4]
        _ = self.s_plotter.make_psh_figure(trace_ids, labels, padding,
                                           "Main title",
                                           ["0", "1", "2", "3", "4"],
                                           show_plot=True)

    def test_make_cycle_figure(self):
        trace_ids = [0, 1, 2, 3, 4]
        forward_shift = 3
        _ = self.s_plotter.make_cycle_figure(trace_ids,
                                             "Main title",
                                             ["0", "1", "2", "3", "4"],
                                             forward_shift=forward_shift,
                                             show_plot=True)

    def test_make_avg_act_scat_figure(self):
        labels = ["d2", "d3", "d5"]
        _ = self.s_plotter.make_avg_act_scat_figure(labels, "Title",
                                                    figure_layout=[1, 3],
                                                    figsize=(12, 10), dpi=160,
                                                    show_plot=True)
        labels = ["d2", "d3", "d5", "d2"]
        _ = self.s_plotter.make_avg_act_scat_figure(labels, "Title",
                                                    figure_layout=[2, 3],
                                                    figsize=(12, 10), dpi=160,
                                                    show_plot=True)


if __name__ == "__main__":
    unittest.main()



from tifffile import TiffFile, imread, imsave
import numpy as np
import json
import os

import matplotlib.pyplot as plt
import warnings
from tqdm import tqdm
import pandas as pd
import PyPDF2


def extract_windows(array, window_size):
    """
    breaks ND array into windows along 0 dimention, starting from the begining and till the end.
    """
    start = 0
    num_windows = array.shape[0] - window_size + 1

    sub_windows = (
            start +
            np.expand_dims(np.arange(window_size), 0) +
            np.expand_dims(np.arange(num_windows), 0).T
    )
    return array[sub_windows]


def sort_by_len0(zip_to_sort):
    """
    Sorts the zip based on the length and alphabet of the first element in zip_to_sort
    """
    # sorts a list based on the length of the first element in zip
    sorted_zip = sorted(zip_to_sort, key=lambda x: (len(x[0])), reverse=True)
    return sorted_zip


def get_baseline(array, window_size, percentile):
    """
    returns average baseline of an ND array along the 0 dimention.
    window_size must be an odd number.

    The baseline for the first and the last window//2 elements is the same :
    the first or the last value calculated with the full window
    """
    mean_signal = np.percentile(extract_windows(array, window_size), percentile, axis=1)
    # construct the baseline
    start = window_size // 2
    end = len(array) - window_size // 2
    # add the beginning and end to baseline
    baseline = np.zeros(array.shape)
    baseline[0:start] = mean_signal[0]
    baseline[start:end] = mean_signal
    baseline[end:] = mean_signal[-1]

    return baseline, start, end


def get_dff(array, window_size):
    """
    subtracts average baseline from an ND array along the 0 dimention.
    window_size must be an odd number.

    The baseline for the first and the last window//2 elements is the same :
    the first or the last value calculated with the full window
    """
    percentile = 8  # 8th percentile
    baseline, start, end = get_baseline(array, window_size, percentile)

    # not to divide by zero:
    b_zero = baseline == 0
    if np.sum(b_zero) > 0:
        warnings.warn(f"{np.sum(b_zero)} baseline values are zero.\n"
                      f"Setting these values to 10^(-6) to avoid dividing by zero.")
        baseline[b_zero] = 10 ^ (-6)

    return (array - baseline) / baseline, start, end


def get_t_score(movie1, movie2, absolute=True):
    """
    Returns absolute t-score image ( 2D or 3D, depending on input),
    for t-score calculations see for example :
    https://www.jmp.com/en_us/statistics-knowledge-portal/t-test/two-sample-t-test.html
    """
    # TODO : check input dimensions

    avg1 = np.mean(movie1, axis=0)
    avg2 = np.mean(movie2, axis=0)

    var1 = np.var(movie1, axis=0)
    var2 = np.var(movie2, axis=0)
    n1 = movie1.shape[0]
    n2 = movie2.shape[0]

    std = np.sqrt(((n1 - 1) * var1 + (n2 - 1) * var2) / (n1 + n2 - 2))

    # not to divide by zero:
    std_zero = std == 0
    if np.sum(std_zero) > 0:
        warnings.warn(f"{np.sum(std_zero)} std values are zero.\n"
                      f"Setting these values to 10^(-6) to avoid dividing by zero.")
        std[std_zero] = 10 ^ (-6)

    if absolute:
        t_score = np.absolute(avg1 - avg2) / std
    else:
        t_score = (avg1 - avg2) / std

    return t_score


def get_diff(movie1, movie2, absolute=False):
    """
    Returns absolute difference image ( 2D or 3D, depending on input).
    per pixel: calculates the mean image for each movie, subtracts and takes the absolute value.
    """
    # TODO : check input dimensions

    avg1 = np.mean(movie1, axis=0)
    avg2 = np.mean(movie2, axis=0)

    if absolute:
        diff = np.absolute(avg1 - avg2)
    else:
        diff = (avg1 - avg2)

    return diff


def plot_errorbar(ax, mean, e, x=None, color='r'):
    if x is None:
        x = np.arange(len(mean))
    ax.errorbar(x, mean, yerr=e, fmt='o', color=color)
    ax.plot(x, mean, color=color)


def get_ax_limits(cycled, mean, e, plot_individual):
    """
    Figures out the tight x and y axis limits
    """
    if plot_individual:
        ymin = np.min(cycled)
        ymax = np.max(cycled)
    else:
        ymin = np.min(mean - e[1, :])
        ymax = np.max(mean + e[0, :])
    xmin = -0.5
    xmax = cycled.shape[1] - 0.5

    return xmin, xmax, ymin, ymax



